@@ -16,20 +16,27 @@
 #  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 #  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
+from __future__ import annotations
 import numpy as np
 from PyQt5.QtCore import Qt
 from PyQt5.QtWidgets import QCheckBox, QComboBox, QLabel, QSizePolicy, QHBoxLayout, QVBoxLayout, QMessageBox
+from magneticalc.QMessageBox2 import QMessageBox2
 from magneticalc.Debug import Debug
-from magneticalc.Groupbox import Groupbox
-from magneticalc.HLine import HLine
-from magneticalc.IconLabel import IconLabel
+from magneticalc.QGroupBox2 import QGroupBox2
+from magneticalc.QHLine import QHLine
+from magneticalc.QIconLabel import QIconLabel
 from magneticalc.SamplingVolume_Widget import SamplingVolume_Widget
-from magneticalc.SliderFloat import SliderFloat
+from magneticalc.QSliderFloat import QSliderFloat
 from magneticalc.Theme import Theme
 from magneticalc.VispyCanvas import VispyCanvas
 
+# Note: Workaround for type hinting
+# noinspection PyUnreachableCode
+if False:
+    from magneticalc.GUI import GUI
 
-class Display_Widget(Groupbox):
+
+class Display_Widget(QGroupBox2):
     """ Display_Widget class. """
 
     # Slider limits
@@ -49,25 +56,26 @@ class Display_Widget(Groupbox):
     # Warn about displaying an excessive number of field labels
     ExcessiveFieldLabelThreshold = 250
 
-    def __init__(self, gui):
+    def __init__(self, gui: GUI) -> None:
         """
         Populates the widget.
 
         @param gui: GUI
         """
-        Groupbox.__init__(self, "Display")
-
+        QGroupBox2.__init__(self, "Display")
+        Debug(self, ": Init")
         self.gui = gui
 
         # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-        self.addWidget(IconLabel("fa.circle", "Point Scale"))
-        self.field_point_scale_slider = SliderFloat(Qt.Horizontal)
+        self.addLayout(QIconLabel("Point Scale", "fa.circle"))
+        self.field_point_scale_slider = QSliderFloat(Qt.Horizontal)
         self.field_point_scale_slider.set_range_step(
             self.FieldPointScaleMinimum,
             self.FieldPointScaleMaximum,
             self.FieldPointScaleStep
         )
+        # noinspection PyUnresolvedReferences
         self.field_point_scale_slider.valueChanged.connect(
             lambda: self.set_field_point_scale(self.field_point_scale_slider.get_value())
         )
@@ -75,9 +83,9 @@ class Display_Widget(Groupbox):
 
         # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-        self.addWidget(HLine())
+        self.addWidget(QHLine())
 
-        self.addWidget(IconLabel("fa.arrow-right", "Arrow Scale"))
+        self.addLayout(QIconLabel("Arrow Scale", "fa.arrow-right"))
 
         field_arrow_scale_layout_left = QVBoxLayout()
         field_arrow_scale_layout_right = QVBoxLayout()
@@ -85,12 +93,13 @@ class Display_Widget(Groupbox):
         field_arrow_head_scale_label = QLabel("Head:")
         field_arrow_head_scale_label.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Expanding)
         field_arrow_scale_layout_left.addWidget(field_arrow_head_scale_label, alignment=Qt.AlignVCenter)
-        self.field_arrow_head_scale_slider = SliderFloat(Qt.Horizontal)
+        self.field_arrow_head_scale_slider = QSliderFloat(Qt.Horizontal)
         self.field_arrow_head_scale_slider.set_range_step(
             self.FieldArrowHeadScaleMinimum,
             self.FieldArrowHeadScaleMaximum,
             self.FieldArrowHeadScaleStep
         )
+        # noinspection PyUnresolvedReferences
         self.field_arrow_head_scale_slider.valueChanged.connect(
             lambda: self.set_field_arrow_head_scale(self.field_arrow_head_scale_slider.get_value())
         )
@@ -99,12 +108,13 @@ class Display_Widget(Groupbox):
         field_arrow_line_scale_label = QLabel("Line:")
         field_arrow_line_scale_label.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Expanding)
         field_arrow_scale_layout_left.addWidget(field_arrow_line_scale_label, alignment=Qt.AlignVCenter | Qt.AlignRight)
-        self.field_arrow_line_scale_slider = SliderFloat(Qt.Horizontal)
+        self.field_arrow_line_scale_slider = QSliderFloat(Qt.Horizontal)
         self.field_arrow_line_scale_slider.set_range_step(
             self.FieldArrowLineScaleMinimum,
             self.FieldArrowLineScaleMaximum,
             self.FieldArrowLineScaleStep
         )
+        # noinspection PyUnresolvedReferences
         self.field_arrow_line_scale_slider.valueChanged.connect(
             lambda: self.set_field_arrow_line_scale(self.field_arrow_line_scale_slider.get_value())
         )
@@ -117,15 +127,16 @@ class Display_Widget(Groupbox):
 
         # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-        self.addWidget(HLine())
+        self.addWidget(QHLine())
 
-        self.addWidget(IconLabel("fa.adjust", "Field Boost"))
-        self.field_boost_slider = SliderFloat(Qt.Horizontal)
+        self.addLayout(QIconLabel("Field Boost", "fa.adjust"))
+        self.field_boost_slider = QSliderFloat(Qt.Horizontal)
         self.field_boost_slider.set_range_step(
             self.FieldBoostMinimum,
             self.FieldBoostMaximum,
             self.FieldBoostStep
         )
+        # noinspection PyUnresolvedReferences
         self.field_boost_slider.valueChanged.connect(
             lambda: self.set_field_boost(self.field_boost_slider.get_value())
         )
@@ -133,10 +144,11 @@ class Display_Widget(Groupbox):
 
         # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-        self.addWidget(HLine())
+        self.addWidget(QHLine())
 
-        self.addWidget(IconLabel("fa.tags", "Field Labels"))
-        self.display_field_magnitude_labels_checkbox = QCheckBox(" Display Magnitude")  # Leading space for alignment
+        self.addLayout(QIconLabel("Field Labels", "fa.tags"))
+        self.display_field_magnitude_labels_checkbox = QCheckBox(" Display Magnitude")
+        # noinspection PyUnresolvedReferences
         self.display_field_magnitude_labels_checkbox.toggled.connect(
             lambda: self.set_display_field_magnitude_labels(self.display_field_magnitude_labels_checkbox.isChecked())
         )
@@ -153,9 +165,9 @@ class Display_Widget(Groupbox):
 
         total_labels_layout = QHBoxLayout()
         total_labels_left = QLabel("Total labels:")
-        total_labels_left.setStyleSheet(f"color: {Theme.LightColor}; font-style: italic;")
+        total_labels_left.setStyleSheet(f"color: {Theme.LiteColor}; font-style: italic;")
         self.total_labels_label = QLabel("N/A")
-        self.total_labels_label.setStyleSheet(f"color: {Theme.PrimaryColor};")
+        self.total_labels_label.setStyleSheet(f"color: {Theme.MainColor};")
         self.total_labels_label.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
         total_labels_layout.addWidget(total_labels_left, alignment=Qt.AlignVCenter)
         total_labels_layout.addWidget(self.total_labels_label, alignment=Qt.AlignVCenter)
@@ -165,7 +177,7 @@ class Display_Widget(Groupbox):
 
         self.reinitialize()
 
-    def reinitialize(self):
+    def reinitialize(self) -> None:
         """
         Re-initializes the widget.
         """
@@ -187,7 +199,7 @@ class Display_Widget(Groupbox):
 
     # ------------------------------------------------------------------------------------------------------------------
 
-    def set_enabled(self, enabled: bool):
+    def set_enabled(self, enabled: bool) -> None:
         """
         Enables / disables this widget.
 
@@ -197,7 +209,7 @@ class Display_Widget(Groupbox):
 
     # ------------------------------------------------------------------------------------------------------------------
 
-    def set_field_point_scale(self, value: float):
+    def set_field_point_scale(self, value: float) -> None:
         """
         Sets field point scale.
 
@@ -209,7 +221,7 @@ class Display_Widget(Groupbox):
         self.gui.config.set_float("field_point_scale", value)
         self.gui.redraw()
 
-    def set_field_arrow_head_scale(self, value: float):
+    def set_field_arrow_head_scale(self, value: float) -> None:
         """
         Sets field arrow head scale.
 
@@ -221,7 +233,7 @@ class Display_Widget(Groupbox):
         self.gui.config.set_float("field_arrow_head_scale", value)
         self.gui.redraw()
 
-    def set_field_arrow_line_scale(self, value: float):
+    def set_field_arrow_line_scale(self, value: float) -> None:
         """
         Sets field arrow line scale.
 
@@ -233,7 +245,7 @@ class Display_Widget(Groupbox):
         self.gui.config.set_float("field_arrow_line_scale", value)
         self.gui.redraw()
 
-    def set_field_boost(self, value: float):
+    def set_field_boost(self, value: float) -> None:
         """
         Sets field boost value.
 
@@ -245,7 +257,7 @@ class Display_Widget(Groupbox):
         self.gui.config.set_float("field_boost", value)
         self.gui.redraw()
 
-    def set_display_field_magnitude_labels(self, value: bool):
+    def set_display_field_magnitude_labels(self, value: bool) -> None:
         """
         Sets field label "Display Magnitude" value.
 
@@ -262,7 +274,7 @@ class Display_Widget(Groupbox):
 
         self.gui.redraw()
 
-    def set_field_label_resolution(self, value: int):
+    def set_field_label_resolution(self, value: int) -> None:
         """
         Sets field label resolution exponent.
 
@@ -279,7 +291,7 @@ class Display_Widget(Groupbox):
 
     # ------------------------------------------------------------------------------------------------------------------
 
-    def update(self):
+    def update(self) -> None:
         """
         Updates this widget.
         """
@@ -288,7 +300,7 @@ class Display_Widget(Groupbox):
         self.update_labels()
         self.update_controls()
 
-    def update_labels(self):
+    def update_labels(self) -> None:
         """
         Updates the labels.
         """
@@ -296,14 +308,14 @@ class Display_Widget(Groupbox):
 
         if self.gui.model.sampling_volume.is_valid():
             n = self.gui.model.sampling_volume.get_labels_count()
-            color = Theme.WarningColor if n > self.ExcessiveFieldLabelThreshold else Theme.LightColor
+            color = Theme.FailureColor if n > self.ExcessiveFieldLabelThreshold else Theme.LiteColor
             self.total_labels_label.setText(str(n))
             self.total_labels_label.setStyleSheet(f"color: {color}; font-style: italic;")
         else:
             self.total_labels_label.setText("N/A")
-            self.total_labels_label.setStyleSheet(f"color: {Theme.LightColor}; font-style: italic;")
+            self.total_labels_label.setStyleSheet(f"color: {Theme.LiteColor}; font-style: italic;")
 
-    def update_controls(self):
+    def update_controls(self) -> None:
         """
         Updates the field label resolution combobox.
         """
@@ -320,6 +332,7 @@ class Display_Widget(Groupbox):
 
         # Re-populate field label resolution combobox
         if self.field_label_resolution_combobox_connection is not None:
+            # noinspection PyUnresolvedReferences
             self.field_label_resolution_combobox.currentIndexChanged.disconnect(
                 self.field_label_resolution_combobox_connection
             )
@@ -336,17 +349,13 @@ class Display_Widget(Groupbox):
                 )
             )
         self.field_label_resolution_combobox_connection = connection
+        # noinspection PyUnresolvedReferences
         self.field_label_resolution_combobox.currentIndexChanged.connect(connection)
 
         # Set default field label resolution if it is not available anymore
         target = self.gui.config.get_int("sampling_volume_label_resolution_exponent")
         if target not in label_resolution_options_dict.values():
-            Debug(
-                self,
-                f": Invalid: sampling_volume_label_resolution_exponent = {target}",
-                color=Theme.WarningColor,
-                force=True
-            )
+            Debug(self, f": WARNING: Invalid: sampling_volume_label_resolution_exponent = {target}", warning=True)
             self.gui.config.set_int(
                 "sampling_volume_label_resolution_exponent",
                 next(iter(label_resolution_options_dict.items()))[1]  # First value from combobox
@@ -395,13 +404,7 @@ class Display_Widget(Groupbox):
             return
 
         if choice:
-
-            messagebox = QMessageBox()
-            messagebox.setWindowTitle("Excessive Number Of Field Labels")
-            messagebox.setIcon(QMessageBox.Question)
-            messagebox.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
-            messagebox.setDefaultButton(QMessageBox.No)
-            messagebox.setText(
+            text = (
                 "You are about to display an excessive number of field labels. "
                 "This will be very slow and cannot be interrupted.\n\n"
                 "DO YOU WANT TO DISPLAY FIELD LABELS ANYWAY?\n\n"
@@ -411,21 +414,29 @@ class Display_Widget(Groupbox):
                 "– Decrease sampling volume resolution.\n"
                 "– Decrease field label resolution."
             )
-            if messagebox.exec() == QMessageBox.No:
+            messagebox = QMessageBox2(
+                title="Excessive Number Of Field Labels",
+                text=text,
+                icon=QMessageBox.Question,
+                buttons=QMessageBox.Yes | QMessageBox.No,
+                default_button=QMessageBox.No
+            )
+            if messagebox.choice == QMessageBox.No:
                 self.disable_field_labels()
 
         else:
 
             self.disable_field_labels()
 
-            messagebox = QMessageBox()
-            messagebox.setWindowTitle("Excessive Number Of Field Labels")
-            messagebox.setIcon(QMessageBox.Information)
-            messagebox.setStandardButtons(QMessageBox.Ok)
-            messagebox.setDefaultButton(QMessageBox.Ok)
-            messagebox.setText(
+            text = (
                 "Field labels were disabled automatically because\n"
                 "an excessive number of field labels was detected.\n\n"
                 "You may manually re-enable field labels after all calculations have finished."
             )
-            messagebox.exec()
+            QMessageBox2(
+                title="Excessive Number Of Field Labels",
+                text=text,
+                icon=QMessageBox.Information,
+                buttons=QMessageBox.Ok,
+                default_button=QMessageBox.Ok
+            )
