@@ -16,15 +16,16 @@
 #  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 #  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
-import os
 import time
 import atexit
-import datetime
 from typing import Optional
-
+from sty import fg
 import qtawesome as qta
+from PyQt5.Qt import QCloseEvent, QKeyEvent
 from PyQt5.QtCore import Qt, QThread, pyqtSignal, QLocale
-from PyQt5.QtWidgets import QMainWindow, QSplitter, QFileDialog, QDesktopWidget, QMessageBox
+from PyQt5.QtWidgets import QMainWindow, QSplitter, QFileDialog, QMessageBox
+from magneticalc.QMessageBox2 import QMessageBox2
+from magneticalc.QSaveAction import QSaveAction
 from magneticalc import API
 from magneticalc.Assert_Dialog import Assert_Dialog
 from magneticalc.CalculationThread import CalculationThread
@@ -44,8 +45,11 @@ from magneticalc.VispyCanvas import VispyCanvas
 class GUI(QMainWindow):
     """ GUI class. """
 
+    # Minimum window size
+    MinimumWindowSize = (800, 600)
+
     # Used by L{Debug}
-    DebugColor = Theme.SuccessColor
+    DebugColor = fg.blue
 
     # Default configuration filename
     DefaultFilename = "MagnetiCalc-DefaultProject.ini"
@@ -54,17 +58,18 @@ class GUI(QMainWindow):
     calculation_status = pyqtSignal(str)
     calculation_exited = pyqtSignal(bool)
 
-    def __init__(self):
+    def __init__(self) -> None:
         """
         Initializes the GUI.
         """
         QMainWindow.__init__(self, flags=Qt.Window)
-
         Debug(self, ": Init")
 
         self.locale = QLocale(QLocale.English)
 
-        self.set_window()
+        self.setWindowIcon(qta.icon("ei.magnet", color=Theme.MainColor))
+        self.setMinimumSize(*self.MinimumWindowSize)
+        self.showMaximized()
 
         self.config = Config()
         self.config.set_changed_callback(self.on_config_changed)
@@ -103,7 +108,9 @@ class GUI(QMainWindow):
         self.menu = Menu(self)
 
         # Connect the calculation thread communication signals
-        self.calculation_status.connect(lambda text: self.statusbar.text(text))
+        # noinspection PyUnresolvedReferences
+        self.calculation_status.connect(lambda text: self.statusbar.set_text(text))
+        # noinspection PyUnresolvedReferences
         self.calculation_exited.connect(lambda success: self.on_calculation_exited(success))
 
         self.initializing = True
@@ -116,26 +123,18 @@ class GUI(QMainWindow):
 
     # ------------------------------------------------------------------------------------------------------------------
 
-    def redraw(self):
+    def redraw(self) -> None:
         """
         Re-draws the scene.
         """
+        Debug(self, ".redraw()")
+
         if self.calculation_thread is not None:
             if self.calculation_thread.isRunning():
-                Debug(
-                    self,
-                    ".redraw(): Skipped because calculation is in progress",
-                    color=Theme.PrimaryColor,
-                    force=True
-                )
+                Debug(self, ".redraw(): WARNING: Skipped because calculation is in progress", warning=True)
                 return
             else:
-                Debug(
-                    self,
-                    ".redraw(): WARNING: Setting calculation thread to None",
-                    color=Theme.WarningColor,
-                    force=True
-                )
+                Debug(self, ".redraw(): WARNING: Setting calculation thread to None", warning=True)
                 self.calculation_thread = None
 
         self.sidebar_right.display_widget.set_enabled(self.model.field.is_valid())
@@ -144,19 +143,14 @@ class GUI(QMainWindow):
 
     # ------------------------------------------------------------------------------------------------------------------
 
-    def recalculate(self):
+    def recalculate(self) -> None:
         """
         Re-calculates the model.
         """
         Debug(self, ".recalculate()")
 
         if self.calculation_thread is not None:
-            Debug(
-                self,
-                ".recalculate(): WARNING: Killing orphaned calculation thread",
-                color=Theme.WarningColor,
-                force=True
-            )
+            Debug(self, ".recalculate(): WARNING: Killing orphaned calculation thread", warning=True)
             self.interrupt_calculation()
 
         if self.initializing:
@@ -171,12 +165,14 @@ class GUI(QMainWindow):
         self.calculation_start_time = time.monotonic()
         self.calculation_thread.start()
 
-    def on_calculation_exited(self, success: bool):
+    def on_calculation_exited(self, success: bool) -> None:
         """
         This is called after calculation thread has exited.
 
         @param success: True if calculation was successful, False otherwise
         """
+        Debug(self, f".on_calculation_exited(success={success})")
+
         calculation_time = time.monotonic() - self.calculation_start_time
 
         if self.calculation_thread is not None:
@@ -189,100 +185,66 @@ class GUI(QMainWindow):
             else:
                 # This happens when calculation finished and no other thread was started
                 self.calculation_thread = None
-                Debug(
-                    self,
-                    f".on_calculation_exited(): Success (took {calculation_time:.2f} s)",
-                    color=Theme.SuccessColor
-                )
+                Debug(self, f".on_calculation_exited(): Success (took {calculation_time:.2f} s)", success=True)
         else:
             Debug(
-                self,
-                f".on_calculation_exited(): Interrupted after {calculation_time:.2f} s", color=Theme.PrimaryColor
-            )
+                self, f".on_calculation_exited(): WARNING: Interrupted after {calculation_time:.2f} s", warning=True)
 
         # Note: For some reason, most of the time we need an additional ("final-final") re-draw here; VisPy glitch?
         self.redraw()
 
         self.statusbar.disarm(success)
 
-    def interrupt_calculation(self):
+    def interrupt_calculation(self) -> None:
         """
         Kills any running calculation.
         """
+        Debug(self, ".interrupt_calculation()")
+
         if self.calculation_thread is None:
-            Debug(
-                self,
-                ".interrupt_calculation: WARNING: No calculation thread to interrupt",
-                color=Theme.WarningColor,
-                force=True
-            )
+            Debug(self, ".interrupt_calculation(): WARNING: No calculation thread to interrupt", warning=True)
             return
 
         if self.calculation_thread.isRunning():
-            Debug(self, ".interrupt_calculation(): Requesting interruption", color=Theme.PrimaryColor)
+            Debug(self, ".interrupt_calculation(): WARNING: Requesting interruption", warning=True)
             self.calculation_thread.requestInterruption()
 
             if self.calculation_thread.wait(5000):
-                Debug(self, ".interrupt_calculation(): Exited gracefully", color=Theme.PrimaryColor)
+                Debug(self, ".interrupt_calculation(): Exited gracefully", success=True)
             else:
                 Assert_Dialog(False, "Failed to terminate calculation thread")
                 if self.calculation_thread is not None:
                     if self.calculation_thread.isRunning():
-                        Debug(
-                            self,
-                            ".interrupt_calculation(): WARNING: Terminating ungracefully",
-                            color=Theme.WarningColor,
-                            force=True
-                        )
+                        Debug(self, ".interrupt_calculation(): WARNING: Terminating ungracefully", warning=True)
                         self.calculation_thread.terminate()
                         self.calculation_thread.wait()
         else:
-            Debug(
-                self,
-                ".interrupt_calculation: WARNING: Calculation thread should be running",
-                color=Theme.WarningColor,
-                force=True
-            )
+            Debug(self, ".interrupt_calculation(): WARNING: Calculation thread should be running", warning=True)
 
         self.calculation_thread = None
 
     # ------------------------------------------------------------------------------------------------------------------
 
-    def set_window(self):
-        """
-        Sets the basic window properties.
-        """
-
-        # Set window icon
-        self.setWindowIcon(qta.icon("ei.magnet", color=Theme.PrimaryColor))
-
-        # Adjust window dimensions to desktop dimensions
-        screen = QDesktopWidget().screenGeometry()
-        self.setGeometry(0, 0, screen.width(), screen.height())
-
-    # ------------------------------------------------------------------------------------------------------------------
-
-    @staticmethod
-    def confirm_saving_unsaved_work(cancelable: bool) -> Optional[bool]:
+    def confirm_saving_unsaved_work(self, cancelable: bool) -> Optional[bool]:
         """
         Confirm saving unsaved work.
 
         @param cancelable: True to make dialog cancelable, False to make dialog non-cancelable
         @return: None if canceled, True if saving, False if discarding
         """
-        messagebox = QMessageBox()
-        messagebox.setWindowTitle("Project Changed")
-        messagebox.setText("Do you want to save your changes?")
-        messagebox.setIcon(QMessageBox.Question)
-        messagebox.setStandardButtons(
-            (QMessageBox.Save | QMessageBox.Discard | QMessageBox.Cancel) if cancelable else
-            (QMessageBox.Save | QMessageBox.Discard)
+        Debug(self, f".confirm_saving_unsaved_work(cancelable={cancelable})")
+
+        buttons = QMessageBox.Save | QMessageBox.Discard | (QMessageBox.Cancel if cancelable else 0)
+        messagebox = QMessageBox2(
+            title="Project Changed",
+            text="Do you want to save your changes?",
+            icon=QMessageBox.Question,
+            buttons=buttons,
+            default_button=QMessageBox.Save
         )
-        messagebox.setDefaultButton(QMessageBox.Save)
-        choice = messagebox.exec()
-        if choice == QMessageBox.Save:
+        if messagebox.choice == QMessageBox.Save:
             return True
-        elif choice == QMessageBox.Discard:
+        elif messagebox.choice == QMessageBox.Discard:
             return False
         else:
             return None
@@ -292,6 +254,8 @@ class GUI(QMainWindow):
         Called by menu "Quit" action.
         Lets user choose to cancel closing or save / discard file if there is unsaved work.
         """
+        Debug(self, ".confirm_close()")
+
         if not self.config.get_synced():
             choice = self.confirm_saving_unsaved_work(cancelable=True)
             if choice is None:
@@ -305,19 +269,22 @@ class GUI(QMainWindow):
 
         self.close()
 
-    def closeEvent(self, _event):
+    def closeEvent(self, _event: QCloseEvent) -> None:
         """
         Handles close event.
 
         @param _event: Close event
         """
         Debug(self, ".closeEvent()")
+
         self.quit()
 
     def quit(self):
         """
         Quits the application.
         """
+        Debug(self, ".quit()")
+
         if self.calculation_thread != QThread.currentThread():
             Debug(self, ".quit()")
             if self.calculation_thread is not None:
@@ -333,7 +300,7 @@ class GUI(QMainWindow):
         # Unregister exit handler (used by Assert_Dialog to exit gracefully)
         atexit.unregister(self.quit)
 
-    def keyPressEvent(self, event):
+    def keyPressEvent(self, event: QKeyEvent) -> None:
         """
         Handles key press event.
 
@@ -371,7 +338,7 @@ class GUI(QMainWindow):
 
     # ------------------------------------------------------------------------------------------------------------------
 
-    def on_config_changed(self):
+    def on_config_changed(self) -> None:
         """
         Gets called when the configuration changed.
         """
@@ -386,10 +353,11 @@ class GUI(QMainWindow):
 
     # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-    def file_open(self):
+    def file_open(self) -> None:
         """
-        Opens some INI file.
+        Opens a project from an INI file.
         """
+        Debug(self, ".file_open()")
 
         # Stop any running calculation
         if self.calculation_thread is not None:
@@ -399,14 +367,14 @@ class GUI(QMainWindow):
 
         filename, _chosen_extension = QFileDialog.getOpenFileName(
             parent=self,
-            caption="Open File",
+            caption="Open Project",
             filter="MagnetiCalc INI File (*.ini)",
             options=QFileDialog.DontUseNativeDialog
         )
 
         if filename != "":
 
-            with ModelAccess(self.gui, recalculate=False):
+            with ModelAccess(self, recalculate=False):
                 self.model.invalidate()
 
             if not self.config.get_synced():
@@ -438,64 +406,61 @@ class GUI(QMainWindow):
 
     # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-    def file_save(self):
+    def file_save(self) -> None:
         """
-        Saves to the currently set INI file.
+        Saves the project to the currently set INI file.
         """
+        Debug(self, ".file_save()")
+
         self.config.save()
 
     # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-    def file_save_as(self):
+    def file_save_as(self) -> None:
         """
-        Saves to some INI file.
+        Saves the project to an INI file.
         """
-        file_dialog = QFileDialog(self)
-        file_dialog.setFileMode(QFileDialog.AnyFile)
-        file_dialog.setWindowTitle("Save As …")
-        file_dialog.setNameFilter("MagnetiCalc INI File (*.ini)")
-        file_dialog.setOptions(QFileDialog.DontUseNativeDialog)
-        if file_dialog.exec():
-            filenames = file_dialog.selectedFiles()
-            if filenames:
-                filename = filenames[0]
+        Debug(self, ".file_save_as()")
 
-                _file_name, file_extension = os.path.splitext(filename)
-
-                if file_extension.lower() != ".ini":
-                    filename += ".ini"
-
-                self.config.set_filename(filename)
-                self.config.save()
-
-    # ------------------------------------------------------------------------------------------------------------------
-
-    def file_save_image(self):
-        """
-        Saves the currently displayed scene to PNG file.
-        """
-        filename, _chosen_extension = QFileDialog.getSaveFileName(
-            parent=self,
-            caption="Save Image",
-            directory=datetime.datetime.now().strftime("%Y-%m-%d_%H-%M-%S_MagnetiCalc"),
-            filter="Portable Network Graphics (*.png)",
-            options=QFileDialog.DontUseNativeDialog
+        action = QSaveAction(
+            self,
+            title="Save Project",
+            date=True,
+            filename="MagnetiCalc_Project",
+            extension=".ini",
+            filter="MagnetiCalc INI File (*.ini)"
         )
-
-        if filename != "":
-            _file_name, file_extension = os.path.splitext(filename)
-
-            if file_extension.lower() != ".png":
-                filename += ".png"
-
-            self.vispy_canvas.save_image(filename)
+        if action.filename:
+            self.config.set_filename(action.filename)
+            self.config.save()
 
     # ------------------------------------------------------------------------------------------------------------------
 
-    def import_wire(self):
+    def file_save_image(self) -> None:
         """
-        Imports wire points from some TXT file.
+        Saves the currently displayed scene to a PNG file.
         """
+        Debug(self, ".file_save_image()")
+
+        action = QSaveAction(
+            self,
+            title="Save Image",
+            date=True,
+            filename="MagnetiCalc_Screenshot",
+            extension=".png",
+            filter="PNG (*.png)"
+        )
+        if action.filename:
+            self.vispy_canvas.save_image(action.filename)
+
+    # ------------------------------------------------------------------------------------------------------------------
+
+    def import_wire(self) -> None:
+        """
+        Imports wire points from a TXT file.
+        """
+        Debug(self, ".import_wire()")
+
         filename, _chosen_extension = QFileDialog.getOpenFileName(
             parent=self,
             caption="Import Wire",
@@ -517,26 +482,23 @@ class GUI(QMainWindow):
                 }
             )
 
-    def export_wire(self):
+    def export_wire(self) -> None:
         """
-        Exports wire points to some TXT file.
+        Exports wire points to a TXT file.
         """
+        Debug(self, ".export_wire()")
+
         if not self.model.wire.is_valid():
             Assert_Dialog(False, "Attempting to export invalid wire")
             return
 
-        filename, _chosen_extension = QFileDialog.getSaveFileName(
-            parent=self,
-            caption="Export Wire",
-            directory=datetime.datetime.now().strftime("%Y-%m-%d_%H-%M-%S_MagnetiCalc_Wire"),
-            filter="Text File (*.txt)",
-            options=QFileDialog.DontUseNativeDialog
+        action = QSaveAction(
+            self,
+            title="Export Wire",
+            date=True,
+            filename="MagnetiCalc_Wire",
+            extension=".txt",
+            filter="Text File (*.txt)"
         )
-
-        if filename != "":
-            _file_name, file_extension = os.path.splitext(filename)
-
-            if file_extension.lower() != ".txt":
-                filename += ".txt"
-
-            API.export_wire(filename, self.model.wire.get_points_sliced())
+        if action.filename:
+            API.export_wire(action.filename, self.model.wire.get_points_sliced())
