@@ -16,42 +16,49 @@
 #  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 #  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
+from __future__ import annotations
 import numpy as np
 import qtawesome as qta
 from si_prefix import si_format
 from PyQt5.QtCore import Qt, QSize
 from PyQt5.QtWidgets import QWidget, QHBoxLayout, QComboBox, QLabel
+from magneticalc.QGroupBox2 import QGroupBox2
+from magneticalc.QHLine import QHLine
+from magneticalc.QIconLabel import QIconLabel
 from magneticalc.Debug import Debug
-from magneticalc.IconLabel import IconLabel
-from magneticalc.Groupbox import Groupbox
-from magneticalc.HLine import HLine
 from magneticalc.Metric import Metric
 from magneticalc.Metric_Presets import Metric_Presets
 from magneticalc.ModelAccess import ModelAccess
 
+# Note: Workaround for type hinting
+# noinspection PyUnreachableCode
+if False:
+    from magneticalc.GUI import GUI
 
-class Metric_Widget(Groupbox):
+
+class Metric_Widget(QGroupBox2):
     """ Metric_Widget class. """
 
     # Formatting settings
     ValuePrecision = 1
 
-    def __init__(self, gui):
+    def __init__(self, gui: GUI) -> None:
         """
         Populates the widget.
 
         @param gui: GUI
         """
-        Groupbox.__init__(self, "Metric")
-
+        QGroupBox2.__init__(self, "Metric")
+        Debug(self, ": Init")
         self.gui = gui
 
         # --------------------------------------------------------------------------------------------------------------
 
-        self.addWidget(IconLabel("fa.tint", "Color"))
+        self.addLayout(QIconLabel("Color", "fa.tint"))
         self.color_metric_combobox = QComboBox()
         for i, preset in enumerate(Metric_Presets.List):
             self.color_metric_combobox.addItem(preset["id"])
+        # noinspection PyUnresolvedReferences
         self.color_metric_combobox.currentIndexChanged.connect(
             lambda: self.set_metric(color_preset=Metric_Presets.get_by_id(self.color_metric_combobox.currentText()))
         )
@@ -77,15 +84,16 @@ class Metric_Widget(Groupbox):
 
         # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-        self.addWidget(HLine())
+        self.addWidget(QHLine())
 
-        self.addWidget(IconLabel("mdi.blur", "Alpha"))
+        self.addLayout(QIconLabel("Alpha", "mdi.blur"))
         self.alpha_metric_combobox = QComboBox()
         for i, preset in enumerate(Metric_Presets.List):
             # Note: (v1.9)
             # Using angle metric for alpha transparency is discouraged and is not available in the combobox anymore.
             if not preset["is_angle"]:
                 self.alpha_metric_combobox.addItem(preset["id"])
+        # noinspection PyUnresolvedReferences
         self.alpha_metric_combobox.currentIndexChanged.connect(
             lambda: self.set_metric(alpha_preset=Metric_Presets.get_by_id(self.alpha_metric_combobox.currentText()))
         )
@@ -111,7 +119,7 @@ class Metric_Widget(Groupbox):
 
         self.reinitialize()
 
-    def reinitialize(self):
+    def reinitialize(self) -> None:
         """
         Re-initializes the widget.
         """
@@ -141,7 +149,7 @@ class Metric_Widget(Groupbox):
             recalculate: bool = True,
             update_labels: bool = True,
             invalidate_self: bool = True
-    ):
+    ) -> None:
         """
         Sets the metric. This will overwrite the currently set metric in the model.
         Any parameter may be left set to None in order to load its default value.
@@ -155,6 +163,8 @@ class Metric_Widget(Groupbox):
         if self.signalsBlocked():
             return
 
+        Debug(self, ".set_metric()")
+
         with ModelAccess(self.gui, recalculate):
 
             # Note: Not using self.gui.config.write_read_str here because we're translating between strings and presets
@@ -206,15 +216,18 @@ class Metric_Widget(Groupbox):
         );
     """
 
-    def update_labels(self):
+    def update_labels(self) -> None:
         """
         Updates the labels.
         """
+        Debug(self, ".update_labels()")
+
         if self.gui.model.metric.is_valid():
 
             limits = self.gui.model.metric.get_limits()
 
-            field_units = self.gui.model.field.get_units()
+            show_gauss = self.gui.config.get_bool("show_gauss")
+            field_units, field_factor = self.gui.model.field.get_units(show_gauss=show_gauss)
 
             if self.gui.model.metric.get_color_preset()["is_angle"]:
                 color_label_min = "0°"
@@ -231,7 +244,7 @@ class Metric_Widget(Groupbox):
                 else:
                     color_label_min = color_log_prefix +\
                         si_format(
-                            limits["color_min"],
+                            limits["color_min"] * field_factor,
                             precision=self.ValuePrecision,
                             exp_format_str="{value}e{expof10} "
                         ) +\
@@ -242,7 +255,7 @@ class Metric_Widget(Groupbox):
                 else:
                     color_label_max = color_log_prefix +\
                         si_format(
-                            limits["color_max"],
+                            limits["color_max"] * field_factor,
                             precision=self.ValuePrecision,
                             exp_format_str="{value}e{expof10} "
                         ) +\
@@ -269,7 +282,7 @@ class Metric_Widget(Groupbox):
                 else:
                     alpha_label_min = alpha_log_prefix +\
                         si_format(
-                            limits["alpha_min"],
+                            limits["alpha_min"] * field_factor,
                             precision=self.ValuePrecision,
                             exp_format_str="{value}e{expof10} "
                         ) +\
@@ -280,7 +293,7 @@ class Metric_Widget(Groupbox):
                 else:
                     alpha_label_max = alpha_log_prefix +\
                         si_format(
-                            limits["alpha_max"],
+                            limits["alpha_max"] * field_factor,
                             precision=self.ValuePrecision,
                             exp_format_str="{value}e{expof10} "
                         ) +\
