@@ -16,26 +16,34 @@
 #  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 #  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
+from __future__ import annotations
+from typing import Dict
 from functools import partial
 from PyQt5.QtCore import Qt
 from PyQt5.QtWidgets import QVBoxLayout, QHBoxLayout, QLabel, QPushButton
-from magneticalc.Groupbox import Groupbox
-from magneticalc.HLine import HLine
+from magneticalc.QGroupBox2 import QGroupBox2
+from magneticalc.QHLine import QHLine
+from magneticalc.Debug import Debug
 from magneticalc.Perspective_Presets import Perspective_Presets
 from magneticalc.Theme import Theme
 
+# Note: Workaround for type hinting
+# noinspection PyUnreachableCode
+if False:
+    from magneticalc.GUI import GUI
 
-class Perspective_Widget(Groupbox):
+
+class Perspective_Widget(QGroupBox2):
     """ Perspective_Widget class. """
 
-    def __init__(self, gui):
+    def __init__(self, gui: GUI) -> None:
         """
         Populates the widget.
 
         @param gui: GUI
         """
-        Groupbox.__init__(self, "Perspective")
-
+        QGroupBox2.__init__(self, "Perspective")
+        Debug(self, ": Init")
         self.gui = gui
 
         # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -46,6 +54,7 @@ class Perspective_Widget(Groupbox):
 
             button = QPushButton(preset["id"])
             button.setStyleSheet("background-color: lightgrey")
+            # noinspection PyUnresolvedReferences
             button.clicked.connect(
                 partial(self.set_perspective, preset)
             )
@@ -56,13 +65,13 @@ class Perspective_Widget(Groupbox):
 
         # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-        self.addWidget(HLine())
+        self.addWidget(QHLine())
 
         hint_label = QLabel("Axis Colors:")
         x_label = QLabel("X")
         y_label = QLabel("Y")
         z_label = QLabel("Z")
-        hint_label.setStyleSheet(f"color: {Theme.LightColor}; font-style: italic;")
+        hint_label.setStyleSheet(f"color: {Theme.LiteColor}; font-style: italic;")
         x_label.setStyleSheet("color: #cc0000; font-weight: bold;")
         y_label.setStyleSheet("color: #00cc00; font-weight: bold;")
         z_label.setStyleSheet("color: #0000cc; font-weight: bold;")
@@ -73,7 +82,7 @@ class Perspective_Widget(Groupbox):
         xyz_hint_layout.addWidget(z_label, alignment=Qt.AlignRight | Qt.AlignVCenter)
         self.addLayout(xyz_hint_layout)
 
-    def set_perspective(self, preset):
+    def set_perspective(self, preset: Dict) -> None:
         """
         Sets display perspective.
 
