@@ -16,29 +16,16 @@
 #  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 #  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
-from PyQt5.QtWidgets import QWidget, QStyle
-
 
 class Theme:
     """ Theme class. """
 
     # Theme colors
-
-    PrimaryColor = "#2c82b8"
-    LightColor = "#555555"
+    MainColor = "#2c82b8"
+    LiteColor = "#555555"
     DarkColor = "#12344a"
-
     SuccessColor = "#2e7d32"
-    WarningColor = "#c62828"
-    InvalidColor = "#a00000"
+    FailureColor = "#c62828"
 
-    @staticmethod
-    def get_icon(widget: QWidget, name: str):
-        """
-        Gets a PyQt5 standard icon by name.
-
-        @param widget: Base QWidget
-        @param name: Name
-        @return: PyQt5 icon
-        """
-        return widget.style().standardIcon(getattr(QStyle, name))
+    # Default font
+    DefaultFontFace = "DejaVu Sans Mono"
