@@ -16,22 +16,18 @@
 #  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 #  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
-import qtawesome as qta
-from PyQt5.QtCore import Qt
-from PyQt5.QtWidgets import QDialog, QVBoxLayout, QHBoxLayout, QTextBrowser, QPushButton
+from magneticalc.QDialog2 import QDialog2
+from magneticalc.QTextBrowser2 import QTextBrowser2
+from magneticalc.Debug import Debug
 from magneticalc.Theme import Theme
 
 
-class Usage_Dialog(QDialog):
+class Usage_Dialog(QDialog2):
     """ Usage_Dialog class. """
 
-    # Window dimensions
-    Width = 800
-    Height = 610
-
     # HTML content
     HTML = f"""
-        <h3 style="color: {Theme.PrimaryColor};">First Steps</h3>
+        <h3 style="color: {Theme.MainColor};">First Steps</h3>
         <ul>
             <li>
                 Go to <b>Wire › Load Preset</b> to select a basic wire shape.<br>
@@ -100,7 +96,7 @@ class Usage_Dialog(QDialog):
         All settings (including your wire shape) are stored in the <code>MagnetiCalc.ini</code> file by default.<br>
         Deleting or renaming this file will restore the default settings.
 
-        <h3 style="color: {Theme.PrimaryColor};">What does MagnetiCalc do?</h3>
+        <h3 style="color: {Theme.MainColor};">What does MagnetiCalc do?</h3>
 
         MagnetiCalc calculates the static magnetic flux density, vector potential, energy, self-inductance
         and magnetic dipole moment of arbitrary coils.
@@ -111,15 +107,15 @@ class Usage_Dialog(QDialog):
 
         <i>Experimental feature:</i> To calculate the energy and self-inductance of permeable (i.e. ferrous)
         materials, different core media can be modeled as regions of variable relative permeability;
-        however, core saturation is currently not modeled, leading to excessive flux density values.
+        however, core saturation is currently not modeled, resulting in excessive flux density values.
 
-        <h3 style="color: {Theme.PrimaryColor};">Who needs MagnetiCalc?</h3>
+        <h3 style="color: {Theme.MainColor};">Who needs MagnetiCalc?</h3>
 
         MagnetiCalc does its job for hobbyists, students, engineers and researchers of magnetic phenomena.
         I designed MagnetiCalc from scratch, because I didn't want to mess around with expensive and/or
         overly complex simulation software whenever I needed to solve a magnetostatic problem.
 
-        <h3 style="color: {Theme.PrimaryColor};">How does it work?</h3>
+        <h3 style="color: {Theme.MainColor};">How does it work?</h3>
 
         The B-field calculation is implemented using the Biot-Savart law [1],
         employing multiprocessing techniques;
@@ -148,53 +144,21 @@ class Usage_Dialog(QDialog):
         [3]: Jackson, Klassische Elektrodynamik, 5. Auflage, S. 252, (5.157).<br>
         [4]: Jackson, Klassische Elektrodynamik, 5. Auflage, S. 216, (5.54).<br>
 
-        <br><br><span style="color: {Theme.PrimaryColor};">
+        <br><br><span style="color: {Theme.MainColor};">
             This and more information about MagnetiCalc can be found in the <code>README.md</code> file and on GitHub.
         </span><br>
         """
 
-    def __init__(self):
+    def __init__(self) -> None:
         """
         Initializes "Usage" dialog.
         """
+        QDialog2.__init__(self, title="Usage", width=850)
+        Debug(self, ": Init")
 
-        QDialog.__init__(self)
-
-        self.setWindowTitle("Usage")
-
-        layout = QVBoxLayout()
-        self.setLayout(layout)
-
-        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-
-        text_browser = QTextBrowser()
-        text_browser.setMinimumWidth(self.Width)
-        text_browser.setMinimumHeight(self.Height)
-        text_browser.setStyleSheet("""
-            background: palette(window);
-            border: none;
-            line-height: 20px;
-        """)
-        text_browser.setOpenExternalLinks(True)
-        text_browser.insertHtml(Usage_Dialog.HTML)
-        text_browser.setFocusPolicy(Qt.NoFocus)
-        cursor = text_browser.textCursor()
-        cursor.setPosition(0)
-        text_browser.setTextCursor(cursor)
-        layout.addWidget(text_browser)
-
-        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-
-        button_box = QHBoxLayout()
-        ok_button = QPushButton(qta.icon("fa.check"), " OK")  # Leading space for alignment
-        ok_button.clicked.connect(self.accept)
-        button_box.addWidget(ok_button)
-        layout.addLayout(button_box)
-
-    # ------------------------------------------------------------------------------------------------------------------
-
-    def show(self):
-        """
-        Shows this dialog.
-        """
-        self.exec()
+        text_browser = QTextBrowser2(html=self.HTML)
+        text_browser.setMinimumHeight(600)
+        self.addWidget(text_browser)
+        self.addButtons({
+            "OK": ("fa.check", self.accept)
+        })
