@@ -18,21 +18,28 @@
 #  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 #  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
+import os
 import sys
-from colorit import bold
+from sty import ef
 from PyQt5.QtWidgets import QApplication
+from magneticalc.Config import get_jit_enabled
 from magneticalc.GUI import GUI
 from magneticalc.Version import Version
 
 
-def main():
+def main() -> None:
     """ MagnetiCalc main function. """
 
+    if sys.platform == "win32":
+        os.system("color")
+
     print()
-    print(bold(Version.String))
+    print(ef.bold + Version.String + ef.rs)
     print(Version.Copyright)
     print(Version.License)
     print()
+    print("JIT is " + ("enabled" if get_jit_enabled() else "disabled"))
+    print()
 
     app = QApplication(sys.argv)
 
